/*
 * aTunes 1.13.3
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.views.dialogs.editPreferences;

import java.awt.GridBagConstraints;

import javax.swing.JFileChooser;
import javax.swing.JLabel;

import net.sourceforge.atunes.gui.views.controls.CustomJFileChooser;
import net.sourceforge.atunes.kernel.modules.state.ApplicationState;
import net.sourceforge.atunes.utils.LanguageTool;

/**
 * The Class DevicePanel.
 */
public class DevicePanel extends PreferencesPanel {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 3331810461314007217L;

    /** The location file chooser. */
    private CustomJFileChooser locationFileChooser;

    /**
     * Instantiates a new device panel.
     */
    public DevicePanel() {
        super(LanguageTool.getString("DEVICE"));
        JLabel label = new JLabel(LanguageTool.getString("DEVICE_DEFAULT_LOCATION"));
        locationFileChooser = new CustomJFileChooser(this, 20, JFileChooser.DIRECTORIES_ONLY);

        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1;
        c.weighty = 1;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.NORTH;
        add(label, c);
        c.gridx = 1;
        add(locationFileChooser, c);
    }

    @Override
    public boolean applyPreferences(ApplicationState state) {
        state.setDefaultDeviceLocation(locationFileChooser.getResult());
        return false;
    }

    /**
     * Sets the default device location.
     * 
     * @param location
     *            the new default device location
     */
    private void setDefaultDeviceLocation(String location) {
        locationFileChooser.setText(location);
    }

    @Override
    public void updatePanel(ApplicationState state) {
        setDefaultDeviceLocation(state.getDefaultDeviceLocation());
    }

    @Override
    public boolean validatePanel() {
        return true;
    }

    @Override
    public void dialogVisibilityChanged(boolean visible) {
        // Do nothing
    }

}
