/*
 * aTunes 1.13.3
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.views.dialogs.editPreferences;

// Layout of the general panel

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Locale;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.SwingConstants;

import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.gui.Fonts;
import net.sourceforge.atunes.gui.LookAndFeelSelector;
import net.sourceforge.atunes.gui.images.themes.ThemePreviewLoader;
import net.sourceforge.atunes.kernel.modules.state.ApplicationState;
import net.sourceforge.atunes.kernel.modules.state.beans.LocaleBean;
import net.sourceforge.atunes.misc.SystemProperties;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;

import org.jdesktop.swingx.combobox.ListComboBoxModel;
import org.jvnet.substance.api.renderers.SubstanceDefaultListCellRenderer;

/**
 * The Class GeneralPanel.
 */
public class GeneralPanel extends PreferencesPanel {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = -9216216930198145476L;

    /** The show title. */
    private JCheckBox showTitle;

    /** The window type. */
    private JComboBox windowType;

    /** The language. */
    private JComboBox language;

    /** The show icon tray. */
    private JCheckBox showIconTray;

    /** The show tray player. */
    private JCheckBox showTrayPlayer;

    /** The use default font. */
    private JCheckBox useDefaultFont;

    /** The use os settings for font smoothing checkbox */
    JCheckBox useOSSettingForFontSmoothing;

    /** The use fontsmoothing checkbox */
    JCheckBox useFontSmoothing;

    /** The theme. */
    JComboBox theme;

    /** The theme preview. */
    JLabel themePreview;

    /**
     * Instantiates a new general panel.
     */
    public GeneralPanel() {
        // Titles are in bold. Label order has been changed, so number don't match any more
        super(LanguageTool.getString("GENERAL"));
        showTitle = new JCheckBox(LanguageTool.getString("SHOW_TITLE"));
        JLabel label = new JLabel(LanguageTool.getString("WINDOW_TYPE"));
        label.setFont(Fonts.GENERAL_FONT_BOLD);
        windowType = new JComboBox(new String[] { LanguageTool.getString("STANDARD_WINDOW"), LanguageTool.getString("MULTIPLE_WINDOW") });
        //JLabel label2 = new JLabel(LanguageTool.getString("CHANGE_WINDOW_TYPE_ON_NEXT_START"));
        JLabel label3 = new JLabel(LanguageTool.getString("LANGUAGE"));
        label3.setFont(Fonts.GENERAL_FONT_BOLD);

        List<Locale> langs = LanguageTool.getLanguages();
        Locale[] array = langs.toArray(new Locale[langs.size()]);
        final Locale currentLocale = ApplicationState.getInstance().getLocale().getLocale();
        Arrays.sort(array, new Comparator<Locale>() {
            @Override
            public int compare(Locale l1, Locale l2) {
                return java.text.Collator.getInstance().compare(l1.getDisplayName(currentLocale), l2.getDisplayName(currentLocale));
            }
        });
        language = new JComboBox(array);
        language.setRenderer(new SubstanceDefaultListCellRenderer() {
            private static final long serialVersionUID = 4124370361802581951L;

            @Override
            public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                if (!(value instanceof Locale)) {
                    throw new IllegalArgumentException("Argument value must be instance of Locale");
                }

                Locale displayingLocale = (Locale) value;
                Locale currentLocale = ApplicationState.getInstance().getLocale().getLocale();

                String name = displayingLocale.getDisplayName(currentLocale);
                name = String.valueOf(name.charAt(0)).toUpperCase(currentLocale) + name.substring(1);
                Component c = super.getListCellRendererComponent(list, name, index, isSelected, cellHasFocus);

                // The name of flag file should be flag_<locale>.png
                // if the name of bundle is MainBundle_<locale>.properties
                String flag = StringUtils.getString("flag_", displayingLocale.toString(), ".png");
                File flagFile = new File(StringUtils.getString(Constants.TRANSLATIONS_DIR, SystemProperties.fileSeparator, flag));
                try {
                    ((JLabel) c).setIcon(new ImageIcon(ImageIO.read(flagFile)));
                } catch (IOException e) {
                    // Don't show any flag
                }
                return c;
            }
        });

        //JLabel label4 = new JLabel(LanguageTool.getString("CHANGE_LANGUAGE_ON_NEXT_START"));
        showIconTray = new JCheckBox(LanguageTool.getString("SHOW_TRAY_ICON"));
        showTrayPlayer = new JCheckBox(LanguageTool.getString("SHOW_TRAY_PLAYER"));
        JLabel label5 = new JLabel(LanguageTool.getString("THEME"));
        label5.setFont(Fonts.GENERAL_FONT_BOLD);
        useDefaultFont = new JCheckBox(LanguageTool.getString("USE_DEFAULT_FONT"));
        useOSSettingForFontSmoothing = new JCheckBox(LanguageTool.getString("USE_OS_SETTINGS_FOR_FONT_SMOOTHING"));
        useOSSettingForFontSmoothing.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                useFontSmoothing.setEnabled(!useOSSettingForFontSmoothing.isSelected());

            }
        });
        useFontSmoothing = new JCheckBox(LanguageTool.getString("USE_FONT_SMOOTHING"));
        theme = new JComboBox(new ListComboBoxModel<String>(LookAndFeelSelector.getListOfSkins()));
        theme.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                String t = (String) theme.getSelectedItem();
                themePreview.setIcon(ThemePreviewLoader.getImage(t));
            }
        });
        themePreview = new JLabel(LanguageTool.getString("PREVIEW"));
        themePreview.setVerticalTextPosition(SwingConstants.TOP);
        themePreview.setHorizontalTextPosition(SwingConstants.CENTER);
        //JLabel label6 = new JLabel(LanguageTool.getString("CHANGE_THEME_ON_NEXT_START"));

        GridBagConstraints c = new GridBagConstraints();
        // First display language
        c.gridy = 0;
        c.gridwidth = 1;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.NORTH;
        c.insets = new Insets(5, 0, 0, 0);
        add(label3, c);
        c.gridx = 1;
        add(language, c);
        //c.gridx = 0;
        //c.gridy = 1;
        //c.gridwidth = 2;
        //c.insets = new Insets(5, 0, 0, 0);
        //add(label4, c);
        c.gridx = 0;
        c.gridy = 2;
        c.insets = new Insets(25, 0, 5, 0);
        add(label, c);
        //c.gridwidth = 1;
        c.gridx = 1;
        c.insets = new Insets(25, 0, 0, 0);
        add(windowType, c);
        //c.gridx = 0;
        //c.gridy = 3;
        //c.gridwidth = 2;
        //c.insets = new Insets(5, 0, 0, 0);
        //add(label2, c);
        c.gridx = 0;
        c.gridy = 4;
        c.weightx = 1;
        //c.gridwidth = 1;
        c.insets = new Insets(5, 0, 0, 0);
        add(showTitle, c);
        c.gridx = 1;
        c.gridy = 4;
        c.weightx = 1;
        c.insets = new Insets(5, 0, 0, 0);
        add(useDefaultFont, c);
        c.gridx = 0;
        c.gridy = 5;
        c.weightx = 1;
        c.insets = new Insets(5, 0, 0, 0);
        add(useOSSettingForFontSmoothing, c);
        c.gridx = 1;
        c.gridy = 5;
        c.gridwidth = 1;
        c.insets = new Insets(5, 0, 0, 0);
        add(useFontSmoothing, c);
        c.gridx = 0;
        c.gridy = 6;
        c.gridwidth = 1;
        c.insets = new Insets(5, 0, 0, 0);
        add(showTrayPlayer, c);
        c.gridx = 1;
        c.gridy = 6;
        c.insets = new Insets(5, 0, 0, 0);
        add(showIconTray, c);
        c.gridx = 0;
        c.gridy = 7;
        c.gridwidth = 2;
        c.insets = new Insets(20, 0, 0, 0);
        add(label5, c);
        c.gridx = 0;
        c.gridy = 8;
        c.weighty = 1;
        c.insets = new Insets(5, 0, 0, 0);
        add(themePreview, c);
        c.gridx = 1;
        c.weighty = 1;
        c.insets = new Insets(40, 0, 0, 0);
        add(theme, c);
        //c.gridy = 9;
        //c.weighty = 0;
        //c.gridx = 0;
        //add(label6, c);
    }

    @Override
    public boolean applyPreferences(ApplicationState state) {
        boolean needRestart = false;

        state.setShowTitle(showTitle.isSelected());

        boolean oldMultipleWindow = state.isMultipleWindow();
        boolean newMultipleWindow = windowType.getSelectedItem().equals(LanguageTool.getString("MULTIPLE_WINDOW"));
        state.setMultipleWindow(newMultipleWindow);
        if (oldMultipleWindow != newMultipleWindow) {
            needRestart = true;
        }

        LocaleBean oldLocale = state.getLocale();
        LocaleBean newLocale = new LocaleBean((Locale) language.getSelectedItem());
        state.setLocale(newLocale);
        if (!oldLocale.equals(newLocale)) {
            needRestart = true;
        }

        Boolean oldIsUseDefaultFont = state.isUseDefaultFont();
        Boolean newIsUseDefaultFont = useDefaultFont.isSelected();
        state.setUseDefaultFont(newIsUseDefaultFont);
        if (!oldIsUseDefaultFont.equals(newIsUseDefaultFont)) {
            needRestart = true;
        }

        Boolean oldIsUseOSSettingsForFontSmoothing = state.isUseOSSettingsForFontSmoothing();
        Boolean newIsUseOSSettingsForFontSmoothing = useOSSettingForFontSmoothing.isSelected();
        state.setUseOSSettingsForFontSmoothing(newIsUseOSSettingsForFontSmoothing);
        if (!oldIsUseOSSettingsForFontSmoothing.equals(newIsUseOSSettingsForFontSmoothing)) {
            needRestart = true;
        }

        Boolean oldIsUseFontSmoothing = state.isUseFontSmoothing();
        Boolean newUseFontSmoothing = useFontSmoothing.isSelected();
        state.setUseFontSmoothing(newUseFontSmoothing);
        if (!oldIsUseFontSmoothing.equals(newUseFontSmoothing)) {
            needRestart = true;
        }

        state.setShowSystemTray(showIconTray.isSelected());
        state.setShowTrayPlayer(showTrayPlayer.isSelected());

        String oldTheme = state.getSkin();
        String newTheme = (String) theme.getSelectedItem();
        state.setSkin(newTheme);
        if (!oldTheme.equals(newTheme)) {
            needRestart = true;
        }

        return needRestart;
    }

    /**
     * Sets the language.
     * 
     * @param language
     *            the new language
     */
    private void setLanguage(Locale language) {
        this.language.setSelectedItem(language);
    }

    /**
     * Sets the show icon tray.
     * 
     * @param show
     *            the new show icon tray
     */
    private void setShowIconTray(boolean show) {
        this.showIconTray.setSelected(show);
    }

    /**
     * Sets the show title.
     * 
     * @param show
     *            the new show title
     */
    private void setShowTitle(boolean show) {
        showTitle.setSelected(show);
    }

    /**
     * Sets the show tray player.
     * 
     * @param show
     *            the new show tray player
     */
    private void setShowTrayPlayer(boolean show) {
        this.showTrayPlayer.setSelected(show);
    }

    /**
     * Sets the theme.
     * 
     * @param t
     *            the new theme
     */
    private void setTheme(String t) {
        theme.setSelectedItem(t);
        themePreview.setIcon(ThemePreviewLoader.getImage(t));
    }

    /**
     * Sets the use default font.
     * 
     * @param use
     *            the new use default font
     */
    private void setUseDefaultFont(boolean use) {
        useDefaultFont.setSelected(use);
    }

    /**
     * Sets if the os setting for font smoothing should be used.
     * 
     * @param use
     *            if the os setting for font smoothing should be used
     */
    private void setUseOSSettingsForFontSmoothing(boolean use) {
        useOSSettingForFontSmoothing.setSelected(use);
    }

    /**
     * Sets the font smoothing.
     * 
     * @param use
     *            if font smoothing should be applied
     */
    private void setUseFontSmoothing(boolean use) {
        useFontSmoothing.setSelected(use);
    }

    /**
     * Sets the window type.
     * 
     * @param type
     *            the new window type
     */
    private void setWindowType(String type) {
        windowType.setSelectedItem(type);
    }

    @Override
    public void updatePanel(ApplicationState state) {
        setShowTitle(state.isShowTitle());
        setWindowType(state.isMultipleWindow() ? LanguageTool.getString("MULTIPLE_WINDOW") : LanguageTool.getString("STANDARD_WINDOW"));
        setLanguage(LanguageTool.getLanguageSelected());
        setShowIconTray(state.isShowSystemTray());
        setShowTrayPlayer(state.isShowTrayPlayer());
        setTheme(state.getSkin());
        setUseDefaultFont(state.isUseDefaultFont());
        setUseFontSmoothing(state.isUseFontSmoothing());
        setUseOSSettingsForFontSmoothing(state.isUseOSSettingsForFontSmoothing());
    }

    @Override
    public boolean validatePanel() {
        return true;
    }

    @Override
    public void dialogVisibilityChanged(boolean visible) {
        // Do nothing
    }

}
