/*
 * aTunes 1.13.3
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.views.dialogs.editPreferences;

import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

import net.sourceforge.atunes.kernel.modules.state.ApplicationState;
import net.sourceforge.atunes.misc.SystemProperties;
import net.sourceforge.atunes.utils.LanguageTool;

/**
 * The Class OSDPanel.
 */
public class OSDPanel extends PreferencesPanel {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 4489293347321979288L;

    /** The animate osd. */
    private JCheckBox animateOSD;

    /** The osd duration. */
    private JComboBox osdDuration;

    /** The osd width. */
    private JComboBox osdWidth;

    /** The osd horizontal alignment */
    private JComboBox osdHorizontalAlignment;

    /** The osd vertical alignment */
    private JComboBox osdVerticalAlignment;

    private static final String LEFT = LanguageTool.getString("LEFT");
    private static final String CENTER = LanguageTool.getString("CENTER");
    private static final String RIGHT = LanguageTool.getString("RIGHT");
    private static final String TOP = LanguageTool.getString("TOP");
    private static final String BOTTOM = LanguageTool.getString("BOTTOM");

    /**
     * Instantiates a new oSD panel.
     */

    public OSDPanel() {
        super(LanguageTool.getString("OSD"));
        animateOSD = new JCheckBox(LanguageTool.getString("ANIMATE_OSD"));
        JLabel label = new JLabel(LanguageTool.getString("OSD_DURATION"));
        osdDuration = new JComboBox(new Integer[] { 2, 4, 6 });
        JLabel label2 = new JLabel(LanguageTool.getString("OSD_WIDTH"));
        osdWidth = new JComboBox(new Integer[] { 400, 500, 600 });
        JLabel label3 = new JLabel(LanguageTool.getString("HORIZONTAL_ALIGNMENT"));
        osdHorizontalAlignment = new JComboBox(new String[] { LEFT, CENTER, RIGHT });
        JLabel label4 = new JLabel(LanguageTool.getString("VERTICAL_ALIGNMENT"));
        osdVerticalAlignment = new JComboBox(new String[] { TOP, CENTER, BOTTOM });
        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1;
        c.gridwidth = 2;
        //c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.FIRST_LINE_START;
        add(animateOSD, c);
        c.weightx = 0;
        c.gridy = 1;
        c.gridwidth = 1;
        c.insets = new Insets(10, 10, 0, 0);
        add(label, c);
        c.gridx = 1;
        c.weightx = 1;
        c.insets = new Insets(5, 10, 0, 0);
        add(osdDuration, c);
        c.gridx = 0;
        c.gridy = 2;
        c.weightx = 0;
        c.insets = new Insets(10, 10, 0, 0);
        add(label2, c);
        c.gridx = 1;
        c.weightx = 1;
        c.insets = new Insets(5, 10, 0, 0);
        add(osdWidth, c);
        c.gridx = 0;
        c.gridy = 3;
        c.weightx = 0;
        c.insets = new Insets(10, 10, 0, 0);
        add(label3, c);
        c.gridx = 1;
        c.weightx = 1;
        c.insets = new Insets(5, 10, 0, 0);
        add(osdHorizontalAlignment, c);
        c.gridx = 0;
        c.gridy = 4;
        c.weighty = 1;
        c.weightx = 0;
        c.insets = new Insets(10, 10, 0, 0);
        add(label4, c);
        c.gridx = 1;
        c.weightx = 1;
        c.insets = new Insets(5, 10, 0, 0);
        add(osdVerticalAlignment, c);
    }

    @Override
    public boolean applyPreferences(ApplicationState state) {
        state.setAnimateOSD(animateOSD.isSelected());
        state.setOsdDuration((Integer) osdDuration.getSelectedItem());
        state.setOsdWidth((Integer) osdWidth.getSelectedItem());
        state.setOsdHorizontalAlignment(getAlignment((String) osdHorizontalAlignment.getSelectedItem()));
        state.setOsdVerticalAlignment(getAlignment((String) osdVerticalAlignment.getSelectedItem()));
        return false;
    }

    /**
     * Sets the animate osd.
     * 
     * @param animate
     *            the new animate osd
     */
    private void setAnimateOSD(boolean animate) {
        animateOSD.setSelected(animate);
    }

    /**
     * Sets the oSD duration.
     * 
     * @param time
     *            the new oSD duration
     */
    private void setOSDDuration(int time) {
        osdDuration.setSelectedItem(time);
    }

    /**
     * Sets the OSD width
     * 
     * @param width
     */
    private void setOSDWidth(int width) {
        osdWidth.setSelectedItem(width);
    }

    /**
     * Sets the OSD horizontal alignment
     * 
     * @param alignment
     */
    private void setOSDHorizontalAlignment(int alignment) {
        if (alignment == SwingConstants.LEFT) {
            osdHorizontalAlignment.setSelectedItem(LEFT);
        } else if (alignment == SwingConstants.CENTER) {
            osdHorizontalAlignment.setSelectedItem(CENTER);
        } else if (alignment == SwingConstants.RIGHT) {
            osdHorizontalAlignment.setSelectedItem(RIGHT);
        }
    }

    /**
     * Returns alignment constant given a localized string
     * 
     * @param alignment
     * @return
     */
    private int getAlignment(String alignment) {
        if (LEFT.equals(alignment)) {
            return SwingConstants.LEFT;
        } else if (RIGHT.equals(alignment)) {
            return SwingConstants.RIGHT;
        } else if (TOP.equals(alignment)) {
            return SwingConstants.TOP;
        } else if (BOTTOM.equals(alignment)) {
            return SwingConstants.BOTTOM;
        } else if (CENTER.equals(alignment)) {
            return SwingConstants.CENTER;
        }
        return 0;
    }

    /**
     * Sets the OSD vertical alignment
     * 
     * @param alignment
     */
    private void setOSDVerticalAlignment(int alignment) {
        if (alignment == SwingConstants.TOP) {
            osdVerticalAlignment.setSelectedItem(TOP);
        } else if (alignment == SwingConstants.CENTER) {
            osdVerticalAlignment.setSelectedItem(CENTER);
        } else if (alignment == SwingConstants.BOTTOM) {
            osdVerticalAlignment.setSelectedItem(BOTTOM);
        }
    }

    private JCheckBox getAnimateOSDCheckBox() {
        return animateOSD;
    }

    @Override
    public void updatePanel(ApplicationState state) {
        setAnimateOSD(state.isAnimateOSD());
        setOSDDuration(state.getOsdDuration());
        setOSDWidth(state.getOsdWidth());
        setOSDHorizontalAlignment(state.getOsdHorizontalAlignment());
        setOSDVerticalAlignment(state.getOsdVerticalAlignment());
        getAnimateOSDCheckBox().setEnabled(!SystemProperties.isJava6Update10OrLater);

    }

    @Override
    public boolean validatePanel() {
        return true;
    }

    @Override
    public void dialogVisibilityChanged(boolean visible) {
        // Do nothing
    }

}
