/*
 * aTunes 1.13.3
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.views.dialogs.editPreferences;

import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;

import net.sourceforge.atunes.kernel.modules.state.ApplicationState;
import net.sourceforge.atunes.utils.LanguageTool;

/**
 * The Class RepositoryPanel.
 */
public class RepositoryPanel extends PreferencesPanel {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 3331810461314007217L;

    /** The refresh time. */
    private JComboBox refreshTime;

    /** The save repository as xml. */
    private JCheckBox saveRepositoryAsXml;

    /**
     * Instantiates a new repository panel.
     */

    public RepositoryPanel() {
        super(LanguageTool.getString("REPOSITORY"));
        JLabel label = new JLabel(LanguageTool.getString("REPOSITORY_REFRESH_TIME"));
        refreshTime = new JComboBox(new Integer[] { 0, 5, 10, 15, 30, 60 });
        saveRepositoryAsXml = new JCheckBox(LanguageTool.getString("SAVE_REPOSITORY_AS_XML"));
        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 0;
        //c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.LINE_START;
        add(label, c);
        c.gridx = 1;
        c.weightx = 1;
        c.insets = new Insets(0, 10, 0, 0);
        add(refreshTime, c);
        c.gridx = 0;
        c.gridy = 1;
        c.weighty = 1;
        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.gridwidth = 2;
        c.insets = new Insets(0, 0, 0, 0);
        add(saveRepositoryAsXml, c);
    }

    @Override
    public boolean applyPreferences(ApplicationState state) {
        state.setAutoRepositoryRefreshTime((Integer) refreshTime.getSelectedItem());
        state.setSaveRepositoryAsXml(saveRepositoryAsXml.isSelected());
        return false;
    }

    /**
     * Sets the refresh time.
     * 
     * @param time
     *            the new refresh time
     */
    private void setRefreshTime(int time) {
        refreshTime.setSelectedItem(time);
    }

    /**
     * Sets the save repository as xml.
     * 
     * @param b
     *            the new save repository as xml
     */
    private void setSaveRepositoryAsXml(boolean b) {
        saveRepositoryAsXml.setSelected(b);
    }

    @Override
    public void updatePanel(ApplicationState state) {
        setRefreshTime(state.getAutoRepositoryRefreshTime());
        setSaveRepositoryAsXml(state.isSaveRepositoryAsXml());
    }

    @Override
    public boolean validatePanel() {
        return true;
    }

    @Override
    public void dialogVisibilityChanged(boolean visible) {
        // Do nothing
    }

}
