/*
 * aTunes 1.13.3
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.actions;

import java.util.HashMap;
import java.util.Map;

import javax.swing.Action;

import net.sourceforge.atunes.misc.log.LogCategories;
import net.sourceforge.atunes.misc.log.Logger;

public class Actions {

    /**
     * Common logger for actions
     */
    private static Logger logger = new Logger();

    /**
     * Map of actions. All actions are singleton
     */
    private static Map<Class<? extends Action>, Action> map = new HashMap<Class<? extends Action>, Action>();

    /**
     * Returns an action object
     * 
     * @param clazz
     * @return
     */
    public static Action getAction(Class<? extends Action> clazz) {
        Action action = map.get(clazz);
        if (action == null) {
            try {
                action = clazz.newInstance();
                map.put(clazz, action);
            } catch (InstantiationException e) {
                logger.internalError(e);
            } catch (IllegalAccessException e) {
                logger.internalError(e);
            }
        }
        return action;
    }

    /**
     * Method to log error from actions
     * 
     * @param obj
     */
    protected static void error(Object obj) {
        logger.error(LogCategories.ACTION, obj);
    }

    /**
     * Enables or disables a group of actions
     * 
     * @param enable
     * @param actions
     */
    public static void setEnabledActions(boolean enable, Class<? extends Action>... actions) {
        for (Class<? extends Action> action : actions) {
            getAction(action).setEnabled(enable);
        }
    }
}
