/*
 * aTunes 1.13.3
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.fileProperties;

import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.concurrent.ExecutionException;

import javax.swing.ImageIcon;
import javax.swing.SwingWorker;

import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.views.panels.FilePropertiesPanel;
import net.sourceforge.atunes.kernel.controllers.model.PanelController;
import net.sourceforge.atunes.kernel.modules.favorites.FavoritesHandler;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.visual.VisualHandler;
import net.sourceforge.atunes.misc.log.LogCategories;
import net.sourceforge.atunes.misc.log.Logger;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.AudioFilePictureUtils;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.TimeUtils;

/**
 * The Class FilePropertiesController.
 * 
 * @author fleax
 */
public class FilePropertiesController extends PanelController<FilePropertiesPanel> {

    /** The current file. */
    AudioFile currentFile;

    /**
     * Instantiates a new file properties controller.
     * 
     * @param panelControlled
     *            the panel controlled
     */
    public FilePropertiesController(FilePropertiesPanel pControlled) {
        super(pControlled);
        addBindings();
        addStateBindings();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * net.sourceforge.atunes.kernel.controllers.model.Controller#addBindings()
     */
    @Override
    protected void addBindings() {
        getPanelControlled().getPictureLabel().addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
                VisualHandler.getInstance().showImageDialog(currentFile);
            }
        });
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * net.sourceforge.atunes.kernel.controllers.model.Controller#addStateBindings
     * ()
     */
    @Override
    protected void addStateBindings() {
        // Nothing to do
    }

    /**
     * Clear.
     */
    public void clear() {
        currentFile = null;
        getPanelControlled().getPictureLabel().setIcon(null);
        getPanelControlled().getPictureLabel().setVisible(false);
        getPanelControlled().getSongPanel().setVisible(false);
    }

    /**
     * Fill file properties.
     */

    void fillFileProperties() {
        if (currentFile != null) {
            getPanelControlled().getBitrateLabel().setText(StringUtils.getString("<html><b>", LanguageTool.getString("BITRATE"), ":</b>    ", currentFile.getBitrate(), " Kbps"));
            getPanelControlled().getFrequencyLabel().setText(
                    StringUtils.getString("<html><b>", LanguageTool.getString("FREQUENCY"), ":</b>    ", currentFile.getFrequency(), " Hz"));
        } else {
            getPanelControlled().getBitrateLabel().setText(StringUtils.getString("<html><b>", LanguageTool.getString("BITRATE"), ":</b>    "));
            getPanelControlled().getFrequencyLabel().setText(StringUtils.getString("<html><b>", LanguageTool.getString("FREQUENCY"), ":</b>    "));
        }

    }

    /**
     * Fill picture.
     */
    void fillPicture() {
        if (currentFile != null) {
            new SwingWorker<ImageIcon, Void>() {
                @Override
                protected ImageIcon doInBackground() throws Exception {
                    return AudioFilePictureUtils.getImageForAudioFile(currentFile, Constants.IMAGE_WIDTH, Constants.IMAGE_HEIGHT);
                }

                @Override
                protected void done() {
                    try {
                        if (get() != null) {
                            getPanelControlled().getPictureLabel().setPreferredSize(new Dimension(get().getIconWidth(), get().getIconHeight()));
                            getPanelControlled().getPictureLabel().setIcon(get());
                            getPanelControlled().getPictureLabel().setVisible(true);
                        } else {
                            getPanelControlled().getPictureLabel().setIcon(null);
                            getPanelControlled().getPictureLabel().setVisible(false);
                        }
                    } catch (InterruptedException e) {
                        getLogger().internalError(e);
                    } catch (ExecutionException e) {
                        getLogger().internalError(e);
                    }
                }
            }.execute();
        }

    }

    /**
     * Returns super logger
     */
    @Override
    protected Logger getLogger() {
        return super.getLogger();
    }

    /**
     * Fill song properties.
     */

    void fillSongProperties() {
        if (currentFile != null && currentFile.getTag() != null) {
            long size = currentFile.getFile().length();
            // \u202D is Unicode symbol for orientation override to LTR. 
            getPanelControlled().getFileNameLabel().setText(
                    StringUtils.getString("<html><b>", LanguageTool.getString("FILE"), ":</b>    ", "\u202D", currentFile.getUrl(), " (", StringUtils.fromByteToMegaOrGiga(size),
                            ") \u202C </html>"));

            getPanelControlled().getSongLabel().setText(
                    StringUtils.getString("<html><b>", LanguageTool.getString("SONG"), ":</b>    ", currentFile.getTitleOrFileName(), " - ", currentFile.getArtist(), " - ",
                            currentFile.getAlbum(), "\u202D (", TimeUtils.seconds2String(currentFile.getDuration()), ") \u202C </html>"));

            if (currentFile.getTag().getTrackNumber() > 0) {
                getPanelControlled().getTrackLabel().setText(
                        StringUtils.getString("<html><b>", LanguageTool.getString("TRACK"), ":</b>    ", currentFile.getTag().getTrackNumber()));
            } else {
                getPanelControlled().getTrackLabel().setText(StringUtils.getString("<html><b>", LanguageTool.getString("TRACK"), ":"));
            }

            if (currentFile.getTag().getYear() > 0) {
                getPanelControlled().getYearLabel().setText(StringUtils.getString("<html><b>", LanguageTool.getString("YEAR"), ":</b>    ", currentFile.getTag().getYear()));
            } else {
                getPanelControlled().getYearLabel().setText(StringUtils.getString("<html><b>", LanguageTool.getString("YEAR"), ":"));
            }
            
            if (currentFile.getTag().getDiscNumber() > 0) {
                getPanelControlled().getDiscNumberLabel().setText(StringUtils.getString("<html><b>", LanguageTool.getString("DISC_NUMBER"), ":</b>    ", currentFile.getDiscNumber()));
            } else {
                getPanelControlled().getDiscNumberLabel().setText(StringUtils.getString("<html><b>", LanguageTool.getString("DISC_NUMBER"), ":"));
            }
            
            getPanelControlled().getGenreLabel().setText(StringUtils.getString("<html><b>", LanguageTool.getString("GENRE"), ":</b>    ", currentFile.getTag().getGenre()));

            // Favorite icons
            refreshFavoriteIcons();
        } else {
            getPanelControlled().getFileNameLabel().setText(StringUtils.getString("<html><b>", LanguageTool.getString("FILE"), ":</b>    "));
            getPanelControlled().getSongLabel().setText(StringUtils.getString("<html><b>", LanguageTool.getString("SONG"), ":</b>    "));
            getPanelControlled().getTrackLabel().setText(StringUtils.getString("<html><b>", LanguageTool.getString("TRACK"), ":</b>    "));
            getPanelControlled().getYearLabel().setText(StringUtils.getString("<html><b>", LanguageTool.getString("YEAR"), ":</b>    "));
            getPanelControlled().getGenreLabel().setText(StringUtils.getString("<html><b>", LanguageTool.getString("GENRE"), ":</b>    "));
            getPanelControlled().getDiscNumberLabel().setText(StringUtils.getString("<html><b>", LanguageTool.getString("DISC_NUMBER"), ":</b>    "));
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * net.sourceforge.atunes.kernel.controllers.model.Controller#notifyReload()
     */
    @Override
    protected void notifyReload() {
        // Nothing to do
    }

    /**
     * Only show properties panel.
     */
    public void onlyShowPropertiesPanel() {
        currentFile = null;
        // Song properties
        fillSongProperties();
        // File Properties
        fillFileProperties();
        // Picture
        fillPicture();
        getPanelControlled().getSongPanel().setVisible(true);
    }

    /**
     * Refresh favorite icons.
     */
    public void refreshFavoriteIcons() {
        if (currentFile == null) {
            return;
        }
        boolean favorite = FavoritesHandler.getInstance().getFavoriteSongsInfo().containsValue(currentFile)
                || FavoritesHandler.getInstance().getFavoriteArtistsInfo().containsKey(currentFile.getArtist())
                || FavoritesHandler.getInstance().getFavoriteAlbumsInfo().containsKey(currentFile.getAlbum());

        getPanelControlled().getSongLabel().setIcon(favorite ? ImageLoader.FAVORITE : null);
    }

    /**
     * Refresh picture.
     */
    public void refreshPicture() {
        fillPicture();
    }

    //TODO
    /**
     * Update values.
     * 
     * @param file
     *            the file
     */
    public void updateValues(AudioObject file) {
        if (file == null) {
            clear();
            return;
        }
        getLogger().debug(LogCategories.CONTROLLER, new String[] { file.getUrl() });
        if (!(file instanceof AudioFile)) {
            clear();
            return;
        }
        currentFile = (AudioFile) file;
        // Song properties
        fillSongProperties();
        // File Properties
        fillFileProperties();
        // Picture
        fillPicture();
        getPanelControlled().getSongPanel().setVisible(true);
    }

    /**
     * Returns panel controlled
     * 
     * @return
     */
    @Override
    protected FilePropertiesPanel getPanelControlled() {
        return super.getPanelControlled();
    }

}
