/*
 * aTunes 1.13.3
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.navigation;

import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import net.sourceforge.atunes.kernel.modules.navigator.NavigationViews;
import net.sourceforge.atunes.kernel.modules.navigator.RepositoryNavigationView;
import net.sourceforge.atunes.kernel.modules.repository.model.Album;
import net.sourceforge.atunes.kernel.modules.state.ApplicationState;
import net.sourceforge.atunes.misc.log.Logger;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;

/**
 * The listener interface for receiving navigationTreeToolTip events.
 */
public class NavigationTreeToolTipListener implements MouseMotionListener, MouseWheelListener {

    /** The Constant logger. */
    static final Logger logger = new Logger();

    /** The controller. */
    NavigationController controller;

    /** The album. */
    Album album;

    /**
     * Instantiates a new navigation tree tool tip listener.
     * 
     * @param controller
     *            the controller
     * @param panel
     *            the panel
     */
    public NavigationTreeToolTipListener(NavigationController controller) {
        this.controller = controller;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent
     * )
     */
    @Override
    public void mouseDragged(MouseEvent arg0) {
        if (!ApplicationState.getInstance().isShowAlbumTooltip()) {
            return;
        }

        controller.setCurrentAlbumToolTipContent(null);
        controller.getAlbumToolTip().setVisible(false);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
     */
    @Override
    public void mouseMoved(MouseEvent e) {
        if (!ApplicationState.getInstance().isShowAlbumTooltip()) {
            return;
        }

        TreePath selectedPath = NavigationViews.getView(RepositoryNavigationView.class).getTree().getPathForLocation(e.getX(), e.getY());
        if (selectedPath != null) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) selectedPath.getLastPathComponent();
            final Object content = node.getUserObject();

            if (content.equals(album)) {
                return;
            }

            if (content instanceof Album) {
                album = (Album) content;
                if (!controller.getAlbumToolTip().isVisible() || controller.getCurrentAlbumToolTipContent() == null || controller.getCurrentAlbumToolTipContent() != content) {
                    if (controller.getAlbumToolTip().isVisible()) {
                        controller.getAlbumToolTip().setVisible(false);
                    }
                    controller.getAlbumToolTip().setLocation((int) NavigationViews.getView(RepositoryNavigationView.class).getTree().getLocationOnScreen().getX() + e.getX(),
                            (int) NavigationViews.getView(RepositoryNavigationView.class).getTree().getLocationOnScreen().getY() + e.getY() + 20);

                    controller.getAlbumToolTip().setAlbum(((Album) content).getName());
                    controller.getAlbumToolTip().setArtist(((Album) content).getArtist().toString());
                    int songs = controller.getAudioObjectsForTreeNode(NavigationViews.getCurrentView().getClass(), node).size();
                    controller.getAlbumToolTip().setSongs(StringUtils.getString(songs, " ", (songs > 1 ? LanguageTool.getString("SONGS") : LanguageTool.getString("SONG"))));
                    controller.setCurrentAlbumToolTipContent(content);
                    controller.getAlbumToolTip().setPicture(null);

                    controller.getToolTipTimer().setInitialDelay(ApplicationState.getInstance().getAlbumTooltipDelay() * 1000);
                    controller.getToolTipTimer().setRepeats(false);
                    controller.getToolTipTimer().start();
                }
            } else {
                controller.setCurrentAlbumToolTipContent(null);
                controller.getAlbumToolTip().setVisible(false);
                controller.getToolTipTimer().stop();
            }
        } else {
            controller.getAlbumToolTip().setVisible(false);
            controller.getToolTipTimer().stop();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seejava.awt.event.MouseWheelListener#mouseWheelMoved(java.awt.event.
     * MouseWheelEvent)
     */
    @Override
    public void mouseWheelMoved(MouseWheelEvent e) {
        if (!ApplicationState.getInstance().isShowAlbumTooltip()) {
            return;
        }

        controller.setCurrentAlbumToolTipContent(null);
        controller.getAlbumToolTip().setVisible(false);
    }

}
