/*
 * aTunes 1.13.3
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.navigator;

import java.awt.Component;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.ToolTipManager;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import net.sourceforge.atunes.gui.ColorDefinitions;
import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.model.NavigationTableModel;
import net.sourceforge.atunes.gui.model.NavigationTableModel.Property;
import net.sourceforge.atunes.gui.views.controls.DragSourceTree;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.actions.Actions;
import net.sourceforge.atunes.kernel.actions.AddToPlayListAction;
import net.sourceforge.atunes.kernel.actions.ClearTagAction;
import net.sourceforge.atunes.kernel.actions.CopyToDeviceAction;
import net.sourceforge.atunes.kernel.actions.EditTagFromNavigatorAction;
import net.sourceforge.atunes.kernel.actions.EditTitlesAction;
import net.sourceforge.atunes.kernel.actions.ExtractPictureAction;
import net.sourceforge.atunes.kernel.actions.FilterNavigatorAction;
import net.sourceforge.atunes.kernel.actions.OpenFolderFromNavigatorAction;
import net.sourceforge.atunes.kernel.actions.PlayNowAction;
import net.sourceforge.atunes.kernel.actions.RemoveFromDiskAction;
import net.sourceforge.atunes.kernel.actions.RenameAudioFileInNavigationTableAction;
import net.sourceforge.atunes.kernel.actions.SearchArtistAction;
import net.sourceforge.atunes.kernel.actions.SearchArtistAtAction;
import net.sourceforge.atunes.kernel.actions.SetAsPlayListAction;
import net.sourceforge.atunes.kernel.actions.SetFavoriteAlbumFromNavigatorAction;
import net.sourceforge.atunes.kernel.actions.SetFavoriteArtistFromNavigatorAction;
import net.sourceforge.atunes.kernel.actions.SetFavoriteSongFromNavigatorAction;
import net.sourceforge.atunes.kernel.actions.ShowNavigatorTableItemInfoAction;
import net.sourceforge.atunes.kernel.controllers.navigation.NavigationController.ViewMode;
import net.sourceforge.atunes.kernel.modules.device.DeviceHandler;
import net.sourceforge.atunes.kernel.modules.favorites.FavoritesHandler;
import net.sourceforge.atunes.kernel.modules.repository.HighlightFoldersByIncompleteTags;
import net.sourceforge.atunes.kernel.modules.repository.RepositoryHandler;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.model.Album;
import net.sourceforge.atunes.kernel.modules.repository.model.Artist;
import net.sourceforge.atunes.kernel.modules.repository.model.Folder;
import net.sourceforge.atunes.kernel.modules.repository.model.Genre;
import net.sourceforge.atunes.kernel.modules.state.ApplicationState;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.model.TreeObject;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;

import org.jvnet.substance.api.renderers.SubstanceDefaultTreeCellRenderer;

public class RepositoryNavigationView extends NavigationView {

    private static JTree tree;

    private JPopupMenu treePopupMenu;

    private JPopupMenu tablePopupMenu;

    @Override
    public ImageIcon getIcon() {
        return ImageLoader.AUDIO_FILE;
    }

    @Override
    public String getTitle() {
        return LanguageTool.getString("REPOSITORY");
    }

    @Override
    public String getTooltip() {
        return LanguageTool.getString("REPOSITORY_TAB_TOOLTIP");
    }

    @Override
    public JTree getTree() {
        if (tree == null) {
            tree = new DragSourceTree(new DefaultTreeModel(new DefaultMutableTreeNode(LanguageTool.getString("REPOSITORY"))));
            tree.setToggleClickCount(0);
            tree.setCellRenderer(getTreeRenderer());
            ToolTipManager.sharedInstance().registerComponent(tree);
        }
        return tree;
    }

    @Override
    public boolean isAudioObjectsFromNodeNeedSort() {
        return true;
    }

    @Override
    public JPopupMenu getTreePopupMenu() {
        if (treePopupMenu == null) {
            treePopupMenu = new JPopupMenu();
            treePopupMenu.add(Actions.getAction(AddToPlayListAction.class));
            treePopupMenu.add(Actions.getAction(SetAsPlayListAction.class));
            treePopupMenu.add(new JSeparator());
            treePopupMenu.add(Actions.getAction(OpenFolderFromNavigatorAction.class));
            treePopupMenu.add(new JSeparator());
            treePopupMenu.add(Actions.getAction(EditTagFromNavigatorAction.class));
            treePopupMenu.add(Actions.getAction(EditTitlesAction.class));
            treePopupMenu.add(Actions.getAction(ClearTagAction.class));
            treePopupMenu.add(new JSeparator());
            treePopupMenu.add(Actions.getAction(RemoveFromDiskAction.class));
            treePopupMenu.add(new JSeparator());
            treePopupMenu.add(Actions.getAction(CopyToDeviceAction.class));
            treePopupMenu.add(new JSeparator());
            treePopupMenu.add(Actions.getAction(SetFavoriteAlbumFromNavigatorAction.class));
            treePopupMenu.add(Actions.getAction(SetFavoriteArtistFromNavigatorAction.class));
            treePopupMenu.add(new JSeparator());
            treePopupMenu.add(Actions.getAction(SearchArtistAction.class));
            treePopupMenu.add(Actions.getAction(SearchArtistAtAction.class));
            treePopupMenu.add(new JSeparator());
            treePopupMenu.add(Actions.getAction(FilterNavigatorAction.class));
        }
        return treePopupMenu;
    }

    @Override
    public JPopupMenu getTablePopupMenu() {
        if (tablePopupMenu == null) {
            tablePopupMenu = new JPopupMenu();
            tablePopupMenu.add(Actions.getAction(AddToPlayListAction.class));
            tablePopupMenu.add(Actions.getAction(SetAsPlayListAction.class));
            tablePopupMenu.add(Actions.getAction(PlayNowAction.class));
            tablePopupMenu.add(new JSeparator());
            tablePopupMenu.add(Actions.getAction(ShowNavigatorTableItemInfoAction.class));
            tablePopupMenu.add(new JSeparator());
            tablePopupMenu.add(Actions.getAction(OpenFolderFromNavigatorAction.class));
            tablePopupMenu.add(new JSeparator());
            tablePopupMenu.add(Actions.getAction(EditTagFromNavigatorAction.class));
            tablePopupMenu.add(Actions.getAction(ClearTagAction.class));
            tablePopupMenu.add(Actions.getAction(ExtractPictureAction.class));
            tablePopupMenu.add(new JSeparator());
            tablePopupMenu.add(Actions.getAction(RemoveFromDiskAction.class));
            tablePopupMenu.add(Actions.getAction(RenameAudioFileInNavigationTableAction.class));
            tablePopupMenu.add(new JSeparator());
            tablePopupMenu.add(Actions.getAction(CopyToDeviceAction.class));
            tablePopupMenu.add(new JSeparator());
            tablePopupMenu.add(Actions.getAction(SetFavoriteSongFromNavigatorAction.class));
            tablePopupMenu.add(Actions.getAction(SetFavoriteAlbumFromNavigatorAction.class));
            tablePopupMenu.add(Actions.getAction(SetFavoriteArtistFromNavigatorAction.class));
            tablePopupMenu.add(new JSeparator());
            tablePopupMenu.add(Actions.getAction(SearchArtistAction.class));
            tablePopupMenu.add(Actions.getAction(SearchArtistAtAction.class));
            tablePopupMenu.add(new JSeparator());
            tablePopupMenu.add(Actions.getAction(FilterNavigatorAction.class));
        }
        return tablePopupMenu;
    }

    @Override
    protected Map<String, ?> getViewData(ViewMode viewMode) {
        if (viewMode == ViewMode.GENRE) {
            return RepositoryHandler.getInstance().getGenreStructure();
        } else if (viewMode == ViewMode.FOLDER) {
            return RepositoryHandler.getInstance().getFolderStructure();
        } else {
            return RepositoryHandler.getInstance().getArtistAndAlbumStructure();
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    protected void refreshTree(ViewMode viewMode, String treeFilter) {
        debug("Refreshing " + this.getClass().getName());

        // Get model and root
        DefaultTreeModel treeModel = (DefaultTreeModel) getTree().getModel();
        DefaultMutableTreeNode root = (DefaultMutableTreeNode) treeModel.getRoot();

        // Get objects selected before refreshing tree
        List<TreeObject> objectsSelected = RefreshUtils.getTreeObjectsSelected(tree);
        // Get objects expanded before refreshing tree
        List<TreeObject> objectsExpanded = RefreshUtils.getTreeObjectsExpanded(tree, root);

        if (viewMode == ViewMode.GENRE) {
            refreshGenreTree((Map<String, Genre>) getViewData(viewMode), treeFilter, root, treeModel, objectsSelected, objectsExpanded);
        } else if (viewMode == ViewMode.FOLDER) {
            refreshFolderView((Map<String, Folder>) getViewData(viewMode), treeFilter, root, treeModel, objectsSelected, objectsExpanded);
        } else {
            refreshTagView((Map<String, Artist>) getViewData(viewMode), viewMode, treeFilter, root, treeModel, objectsSelected, objectsExpanded);
        }

        getTree().expandRow(0);
    }

    @Override
    public List<AudioObject> getAudioObjectForTreeNode(DefaultMutableTreeNode node, ViewMode viewMode, String treeFilter) {
        List<AudioObject> songs = new ArrayList<AudioObject>();
        if (node.isRoot()) {
            if (treeFilter == null) {
                songs.addAll(RepositoryHandler.getInstance().getAudioFiles());
            } else {
                songs = new ArrayList<AudioObject>();
                for (int i = 0; i < node.getChildCount(); i++) {
                    TreeObject obj = (TreeObject) ((DefaultMutableTreeNode) node.getChildAt(i)).getUserObject();
                    songs.addAll(obj.getAudioObjects());
                }
            }
        } else {
            // If album view is selected, we must work with artist nodes, not album node because
            // several artist can have an album with the same name but only songs of one artist 
            // would be shown.
            if (viewMode == ViewMode.ALBUM) {
                songs = new ArrayList<AudioObject>();
                // Here we have child nodes so it is not the last child (subAlbumNode) that is selected
                if (node.getChildCount() != 0) {
                    for (int i = 0; i < node.getChildCount(); i++) {
                        TreeObject obj = (TreeObject) ((DefaultMutableTreeNode) node.getChildAt(i)).getUserObject();
                        if (node.getChildAt(i).getChildCount() == 0) {
                            songs.addAll(obj.getAudioObjects());
                        } else {
                            // We are in album node and must first open one more node
                            for (int j = 0; j < node.getChildAt(i).getChildCount(); j++) {
                                TreeObject obj2 = (TreeObject) ((DefaultMutableTreeNode) node.getChildAt(i).getChildAt(j)).getUserObject();
                                if (node.getChildAt(i).getChildAt(j).getChildCount() == 0) {
                                    songs.addAll(obj2.getAudioObjects());
                                }
                            }
                        }
                    }
                } else {
                    TreeObject obj = (TreeObject) node.getUserObject();
                    songs = obj.getAudioObjects();
                }
            } else {
                TreeObject obj = (TreeObject) node.getUserObject();
                songs = obj.getAudioObjects();
            }
        }
        return songs;
    }

    /**
     * Refresh.
     * 
     * @param structure
     *            the structure
     * @param tree
     *            the tree
     * @param treeModel
     *            the tree model
     * @param viewMode
     *            the view mode
     * @param currentFilter
     *            the current filter
     */
    private void refreshTagView(Map<String, Artist> structure, ViewMode viewMode, String currentFilter, DefaultMutableTreeNode root, DefaultTreeModel treeModel, List<TreeObject> objectsSelected, List<TreeObject> objectsExpanded) {
        // Set root
        root.setUserObject(LanguageTool.getString("REPOSITORY"));
        root.removeAllChildren();

        List<String> artistNamesList = new ArrayList<String>(structure.keySet());
        if (ApplicationState.getInstance().isUseSmartTagViewSorting() && !ApplicationState.getInstance().isUsePersonNamesArtistTagViewSorting()) {
            Collections.sort(artistNamesList, smartComparator);
        } else if (ApplicationState.getInstance().isUsePersonNamesArtistTagViewSorting()) {
            Collections.sort(artistNamesList, artistNamesComparator);
        } else {
            Collections.sort(artistNamesList, comparator);
        }

        // Nodes to be selected after refresh
        List<DefaultMutableTreeNode> nodesToSelect = new ArrayList<DefaultMutableTreeNode>();
        // Nodes to be expanded after refresh
        List<DefaultMutableTreeNode> nodesToExpand = new ArrayList<DefaultMutableTreeNode>();

        if (viewMode == ViewMode.ARTIST) {
            for (int i = 0; i < artistNamesList.size(); i++) {
                Artist artist = structure.get(artistNamesList.get(i));
                DefaultMutableTreeNode artistNode = new DefaultMutableTreeNode(artist);
                List<String> albumNamesList = new ArrayList<String>(artist.getAlbums().keySet());
                if (ApplicationState.getInstance().isUseSmartTagViewSorting()) {
                    Collections.sort(albumNamesList, smartComparator);
                } else {
                    Collections.sort(albumNamesList, comparator);
                }
                if (currentFilter == null || artist.getName().toUpperCase().contains(currentFilter.toUpperCase())) {
                    for (int j = 0; j < albumNamesList.size(); j++) {
                        Album album = artist.getAlbum(albumNamesList.get(j));
                        DefaultMutableTreeNode albumNode = new DefaultMutableTreeNode(album);
                        artistNode.add(albumNode);
                        // If node was selected before refreshing...
                        if (objectsSelected.contains(albumNode.getUserObject())) {
                            nodesToSelect.add(albumNode);
                        }
                    }
                    root.add(artistNode);
                    // Reload causes very important lag on large collections and if it is not used
                    // selection does not work.

                    // If node was selected before refreshing...
                    if (objectsSelected.contains(artistNode.getUserObject())) {
                        nodesToSelect.add(artistNode);
                    }
                    // If node was expanded before refreshing...
                    if (objectsExpanded.contains(artistNode.getUserObject())) {
                        nodesToExpand.add(artistNode);
                    }
                }
            }
        } else {
            // Album view

            // Gather albums
            Map<String, Album> albums = new HashMap<String, Album>();
            DefaultMutableTreeNode albumNode = null;
            for (int i = 0; i < artistNamesList.size(); i++) {
                Artist artist = structure.get(artistNamesList.get(i));
                List<String> albumNamesList = new ArrayList<String>(artist.getAlbums().keySet());
                Collections.sort(albumNamesList);
                for (int j = 0; j < albumNamesList.size(); j++) {
                    String a = albumNamesList.get(j);
                    Album album = artist.getAlbum(a);
                    if (currentFilter == null || album.getName().toUpperCase().contains(currentFilter.toUpperCase())) {
                        albums.put(album.getName() + artist.toString(), album);
                    }
                }
            }

            // Then add them in the tree with the artists
            List<String> albumsList = new ArrayList<String>(albums.keySet());
            Collections.sort(albumsList, comparator);
            for (int i = 0; i < albumsList.size(); i++) {
                Album a = albums.get(albumsList.get(i));
                DefaultMutableTreeNode artistNode = new DefaultMutableTreeNode(a.getArtist());
                if (i == 0 || !albums.get(albumsList.get(i)).toString().equals((albums.get(albumsList.get(i - 1))).toString())) {
                    albumNode = new DefaultMutableTreeNode(a);
                    root.add(albumNode);
                    // If node was selected before refreshing...
                    if (objectsSelected.contains(albumNode.getUserObject())) {
                        nodesToSelect.add(albumNode);
                    }
                }
                if (albumNode != null) {
                    albumNode.add(artistNode);
                }
                // That one is a workaround because the Artist class returns all audiofiles of an artist and not only 
                // the songs from a specific album, so we add that album again. If you have a better idea you can improve
                DefaultMutableTreeNode subAlbumNode = new DefaultMutableTreeNode(a.getArtist().getAlbum(a.getName()));
                artistNode.add(subAlbumNode);
            }

        }
        // Reload the tree to refresh content
        treeModel.reload();

        // Expand nodes
        RefreshUtils.expandNodes(tree, nodesToExpand);

        // Once tree has been refreshed, select previously selected nodes
        RefreshUtils.selectNodes(tree, nodesToSelect);
    }

    /**
     * Refresh genre tree.
     * 
     * @param structure
     *            the structure
     * @param tree
     *            the tree
     * @param treeModel
     *            the tree model
     * @param currentFilter
     *            the current filter
     */
    private void refreshGenreTree(Map<String, Genre> structure, String currentFilter, DefaultMutableTreeNode root, DefaultTreeModel treeModel, List<TreeObject> objectsSelected, List<TreeObject> objectsExpanded) {
        // Nodes to be selected after refresh
        List<DefaultMutableTreeNode> nodesToSelect = new ArrayList<DefaultMutableTreeNode>();
        // Nodes to be expanded after refresh
        List<DefaultMutableTreeNode> nodesToExpand = new ArrayList<DefaultMutableTreeNode>();

        // Refresh nodes
        root.setUserObject(LanguageTool.getString("REPOSITORY"));
        root.removeAllChildren();
        List<String> genreNamesList = new ArrayList<String>(structure.keySet());
        Collections.sort(genreNamesList, comparator);

        for (int i = 0; i < genreNamesList.size(); i++) {
            Genre genre = structure.get(genreNamesList.get(i));
            if (currentFilter == null || genre.getName().toUpperCase().contains(currentFilter.toUpperCase())) {
                DefaultMutableTreeNode genreNode = new DefaultMutableTreeNode(genre);
                // If node was selected before refreshing...
                if (objectsSelected.contains(genreNode.getUserObject())) {
                    nodesToSelect.add(genreNode);
                }
                // If node was expanded before refreshing...
                if (objectsExpanded.contains(genreNode.getUserObject())) {
                    nodesToExpand.add(genreNode);
                }
                List<String> artistNamesList = new ArrayList<String>(genre.getArtists().keySet());
                Collections.sort(artistNamesList);
                for (int j = 0; j < artistNamesList.size(); j++) {
                    Artist artist = genre.getArtist(artistNamesList.get(j));
                    DefaultMutableTreeNode artistNode = new DefaultMutableTreeNode(artist);
                    List<String> albumNamesList = new ArrayList<String>(artist.getAlbums().keySet());
                    Collections.sort(albumNamesList);
                    for (int k = 0; k < albumNamesList.size(); k++) {
                        Album album = artist.getAlbum(albumNamesList.get(k));
                        DefaultMutableTreeNode albumNode = new DefaultMutableTreeNode(album);
                        artistNode.add(albumNode);
                        genreNode.add(artistNode);
                        // If node was selected before refreshing...
                        if (objectsSelected.contains(artistNode.getUserObject())) {
                            nodesToSelect.add(artistNode);
                        }
                        // If node was selected before refreshing...
                        if (objectsSelected.contains(albumNode.getUserObject())) {
                            nodesToSelect.add(albumNode);
                        }
                        // If node was expanded before refreshing...
                        if (objectsExpanded.contains(artistNode.getUserObject()) && objectsExpanded.contains(((DefaultMutableTreeNode) artistNode.getParent()).getUserObject())) {
                            nodesToExpand.add(artistNode);
                        }
                    }
                }
                root.add(genreNode);
            }
        }

        // Reload the tree to refresh content
        treeModel.reload();

        // Expand nodes
        RefreshUtils.expandNodes(tree, nodesToExpand);

        // Once tree has been refreshed, select previously selected nodes
        RefreshUtils.selectNodes(tree, nodesToSelect);
    }

    /**
     * Refresh folder view
     * 
     * @param structure
     *            the structure
     * @param tree
     *            the tree
     * @param treeModel
     *            the tree model
     * @param currentFilter
     *            the current filter
     */
    private void refreshFolderView(Map<String, Folder> structure, String currentFilter, DefaultMutableTreeNode root, DefaultTreeModel treeModel, List<TreeObject> objectsSelected, List<TreeObject> objectsExpanded) {

        // Refresh nodes
        root.setUserObject(LanguageTool.getString("REPOSITORY"));
        root.removeAllChildren();
        RefreshUtils.addFolderNodes(structure, root, currentFilter, comparator);

        treeModel.reload();

        if (objectsExpanded.isEmpty()) {
            // In folder view root child nodes must be expanded always
            // So when refreshing folder view for first time add these nodes to list of expanded objects
            for (int i = 0; i < root.getChildCount(); i++) {
                TreeNode childNode = root.getChildAt(i);
                objectsExpanded.add((TreeObject) ((DefaultMutableTreeNode) childNode).getUserObject());
            }
        }

        // Get nodes to select after refresh
        List<DefaultMutableTreeNode> nodesToSelect = RefreshUtils.getNodes(root, objectsSelected);

        // Get nodes to expand after refresh
        List<DefaultMutableTreeNode> nodesToExpand = RefreshUtils.getNodes(root, objectsExpanded);

        // Expand nodes
        RefreshUtils.expandNodes(tree, nodesToExpand);

        // Once tree has been refreshed, select previously selected nodes
        RefreshUtils.selectNodes(tree, nodesToSelect);
    }

    @Override
    public Class<?> getNavigatorTableColumnClass(int columnIndex) {
        if (columnIndex == 0) {
            return Property.class;
        } else if (columnIndex == 1) {
            if (ApplicationState.getInstance().getViewMode() != ViewMode.FOLDER) {
                return Integer.class;
            }
            return String.class;
        } else if (columnIndex == 2) {
            if (ApplicationState.getInstance().getViewMode() != ViewMode.FOLDER) {
                return String.class;
            }
            return Long.class;
        } else if (columnIndex == 3) {
            return Long.class;
        } else {
            return Long.class;
        }
    }

    @Override
    public int getNavigatorTableColumnCount() {
        if (ApplicationState.getInstance().getViewMode() != ViewMode.FOLDER) {
            return 4;
        }
        return 3;
    }

    @Override
    public String getNavigatorTableColumnName(int columnIndex) {
        if (ApplicationState.getInstance().getViewMode() == ViewMode.FOLDER) {
            switch (columnIndex) {
            case 0:
                return "";
            case 1:
                return LanguageTool.getString("FILE");
            case 2:
                return LanguageTool.getString("DURATION");
            }
        } else {
            switch (columnIndex) {
            case 0:
                return "";
            case 1:
                return "";
            case 2:
                return LanguageTool.getString("TITLE");
            case 3:
                return LanguageTool.getString("DURATION");
            }
        }
        return "";
    }

    @Override
    public Object getNavigatorTableValueAt(AudioObject audioObject, int columnIndex) {
        if (audioObject instanceof AudioFile) {
            if (columnIndex == 0) {
                return ApplicationState.getInstance().isShowFavoritesInNavigator() && FavoritesHandler.getInstance().getFavoriteSongs().contains(audioObject) ? Property.FAVORITE
                        : Property.NO_PROPERTIES;
            } else if (columnIndex == 1) {
                if (ApplicationState.getInstance().getViewMode() != ViewMode.FOLDER) {
                    return audioObject.getTrackNumber();
                }
                return ((AudioFile) audioObject).getFile().getName();
            } else if (columnIndex == 2) {
                if (ApplicationState.getInstance().getViewMode() != ViewMode.FOLDER) {
                    return audioObject.getTitleOrFileName();
                }
                return audioObject.getDuration();
            } else {
                return audioObject.getDuration();
            }
        }
        return "";
    }

    @Override
    public int getNavigatorTableMaxWidthForColumn(int columnIndex) {
        if (columnIndex == 0) {
            return 20;
        } else if (columnIndex == 1) {
            if (ApplicationState.getInstance().getViewMode() == ViewMode.FOLDER) {
                return -1;
            }
            return 25;
        } else if (columnIndex == 2) {
            if (ApplicationState.getInstance().getViewMode() != ViewMode.FOLDER) {
                return -1;
            }
            return 50;
        } else if (columnIndex == 3) {
            if (ApplicationState.getInstance().getViewMode() == ViewMode.FOLDER) {
                return -1;
            }
            return 50;
        }
        return -1;
    }

    @Override
    public boolean isViewModeSupported() {
        return true;
    }

    @Override
    public List<AudioObject> filterNavigatorTable(List<AudioObject> audioObjects, String filter) {
        // Filter by title
        List<AudioObject> filteredObjects = new ArrayList<AudioObject>();
        for (AudioObject audioObject : audioObjects) {
            if (audioObject.getTitleOrFileName().toLowerCase().contains(filter.toLowerCase())) {
                filteredObjects.add(audioObject);
            }
        }
        return filteredObjects;
    }

    @Override
    public boolean isNavigatorTableFilterSupported() {
        return true;
    }

    @SuppressWarnings("unchecked")
    @Override
    public void updateTreePopupMenuWithTreeSelection(MouseEvent e) {
        boolean rootSelected = getTree().isRowSelected(0);
        TreePath path = getTree().getSelectionPath();

        Actions.getAction(SetAsPlayListAction.class).setEnabled(true);
        Actions.getAction(AddToPlayListAction.class).setEnabled(true);

        //		Actions.setEnabledActions(false, SetFavoriteSongFromNavigatorAction.class, 
        //										 PlayNowAction.class,
        //										 ExtractPictureAction.class);

        Actions.getAction(FilterNavigatorAction.class).setEnabled(true);

        Actions.setEnabledActions(!rootSelected && getTree().getSelectionCount() == 1, EditTagFromNavigatorAction.class, ClearTagAction.class);

        Actions.getAction(RemoveFromDiskAction.class).setEnabled(!rootSelected);
        Actions.getAction(CopyToDeviceAction.class).setEnabled(!rootSelected && DeviceHandler.getInstance().isDeviceConnected());
        Folder selectedFolder = null;
        if (path != null) {
            Object obj = ((DefaultMutableTreeNode) path.getLastPathComponent()).getUserObject();
            if (obj instanceof Folder) {
                selectedFolder = (Folder) obj;
            }
            if (obj instanceof Genre) {
                Actions.setEnabledActions(false, EditTagFromNavigatorAction.class, ClearTagAction.class);
            }
            Actions.getAction(SetFavoriteAlbumFromNavigatorAction.class).setEnabled(obj instanceof Album && ApplicationState.getInstance().getViewMode() != ViewMode.FOLDER);
            Actions.getAction(SetFavoriteArtistFromNavigatorAction.class).setEnabled(obj instanceof Artist && ApplicationState.getInstance().getViewMode() != ViewMode.FOLDER);

            Actions.getAction(EditTitlesAction.class).setEnabled(obj instanceof Album);
            Actions.getAction(SearchArtistAction.class).setEnabled(obj instanceof Artist && !(((Artist) obj).getName().equals(LanguageTool.getString("UNKNOWN_ARTIST"))));
            Actions.getAction(SearchArtistAtAction.class).setEnabled(obj instanceof Artist && !(((Artist) obj).getName().equals(LanguageTool.getString("UNKNOWN_ARTIST"))));
        } else {
            Actions.setEnabledActions(false, EditTitlesAction.class, SearchArtistAction.class, SearchArtistAtAction.class, SetFavoriteAlbumFromNavigatorAction.class,
                    SetFavoriteArtistFromNavigatorAction.class);
        }
        List<AudioFile> filesSelectedInNavigator = ControllerProxy.getInstance().getNavigationController().getFilesSelectedInNavigator();
        Actions.getAction(OpenFolderFromNavigatorAction.class).setEnabled(
                ControllerProxy.getInstance().getNavigationController().sameParentFile(filesSelectedInNavigator) || selectedFolder != null);
    }

    @Override
    public void updateTablePopupMenuWithTableSelection(JTable table, MouseEvent e) {
        Actions.getAction(SetAsPlayListAction.class).setEnabled(true);
        Actions.getAction(AddToPlayListAction.class).setEnabled(true);
        Actions.getAction(PlayNowAction.class).setEnabled(table.getSelectedRowCount() == 1);
        Actions.getAction(RenameAudioFileInNavigationTableAction.class).setEnabled(table.getSelectedRowCount() == 1);
        Actions.getAction(ShowNavigatorTableItemInfoAction.class).setEnabled(table.getSelectedRowCount() == 1);

        AudioObject ao = ((NavigationTableModel) table.getModel()).getSongAt(table.getSelectedRow());
        Actions.getAction(ExtractPictureAction.class).setEnabled(table.getSelectedRowCount() == 1 && ao instanceof AudioFile && ((AudioFile) ao).hasInternalPicture());

        Actions.getAction(EditTitlesAction.class).setEnabled(false);
        Actions.getAction(SearchArtistAtAction.class).setEnabled(false);
        Actions.getAction(EditTagFromNavigatorAction.class).setEnabled(true);
        Actions.getAction(ClearTagAction.class).setEnabled(true);
        Actions.getAction(SetFavoriteSongFromNavigatorAction.class).setEnabled(true);
        Actions.getAction(SearchArtistAction.class).setEnabled(true);
        Actions.getAction(RemoveFromDiskAction.class).setEnabled(true);
        Actions.getAction(FilterNavigatorAction.class).setEnabled(true);

        Actions.getAction(CopyToDeviceAction.class).setEnabled(DeviceHandler.getInstance().isDeviceConnected());

        if (table.getSelectedRowCount() > 0) {
            Actions.getAction(SetFavoriteAlbumFromNavigatorAction.class).setEnabled(false);
            Actions.getAction(SetFavoriteArtistFromNavigatorAction.class).setEnabled(false);
        }

        List<AudioFile> filesSelectedInNavigator = ControllerProxy.getInstance().getNavigationController().getFilesSelectedInNavigator();
        Actions.getAction(OpenFolderFromNavigatorAction.class).setEnabled(ControllerProxy.getInstance().getNavigationController().sameParentFile(filesSelectedInNavigator));
    }

    @Override
    protected TreeCellRenderer getTreeRenderer() {
        return new SubstanceDefaultTreeCellRenderer() {
            private static final long serialVersionUID = -7992021225213275134L;

            @Override
            public Component getTreeCellRendererComponent(JTree tree, Object value, boolean isSelected, boolean expanded, boolean leaf, int row, boolean isHasFocus) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
                final Object content = node.getUserObject();

                JLabel label = (JLabel) super.getTreeCellRendererComponent(tree, value, isSelected, expanded, leaf, row, isHasFocus);
                if (content instanceof Artist) {
                    if (!ApplicationState.getInstance().isShowFavoritesInNavigator() || !FavoritesHandler.getInstance().getFavoriteArtistsInfo().containsValue(content)) {
                        label.setIcon(ImageLoader.ARTIST);
                    } else {
                        label.setIcon(ImageLoader.ARTIST_FAVORITE);
                    }
                    label.setToolTipText(getToolTipForArtist((Artist) content));
                } else if (content instanceof Album) {
                    if (!ApplicationState.getInstance().isShowFavoritesInNavigator() || !FavoritesHandler.getInstance().getFavoriteAlbumsInfo().containsValue(content)) {
                        label.setIcon(ImageLoader.ALBUM);
                    } else {
                        label.setIcon(ImageLoader.ALBUM_FAVORITE);
                    }
                    if (!ApplicationState.getInstance().isShowAlbumTooltip()) {
                        label.setToolTipText(getToolTipForAlbum((Album) content));
                    } else {
                        // If using custom album tooltip we must set tooltip to null. If not will appear the tooltip of the parent node
                        // (the tooltip of the artist)
                        label.setToolTipText(null);
                    }
                } else if (content instanceof Genre) {
                    label.setIcon(ImageLoader.GENRE);
                    label.setToolTipText(getToolTipForGenre((Genre) content));
                } else if (content instanceof Folder) {
                    label.setIcon(ImageLoader.FOLDER);
                    label.setToolTipText(getToolTipForFolder((Folder) content));
                    if (ApplicationState.getInstance().isHighlightIncompleteTagFolders() && HighlightFoldersByIncompleteTags.hasIncompleteTags((Folder) content)) {
                        label.setForeground(ColorDefinitions.GENERAL_UNKNOWN_ELEMENT_FOREGROUND_COLOR);
                    }
                } else if (content instanceof String) {
                    if (((String) content).equals(LanguageTool.getString("REPOSITORY"))) {
                        label.setIcon(ImageLoader.AUDIO_FILE);
                    } else {
                        label.setIcon(ImageLoader.FOLDER);
                    }
                    label.setToolTipText(null);
                }

                if (value.toString() != null) {
                    if (value.toString().equals(LanguageTool.getString("UNKNOWN_ARTIST")) || value.toString().equals(LanguageTool.getString("UNKNOWN_ALBUM"))
                            || value.toString().equals(LanguageTool.getString("UNKNOWN_GENRE"))) {
                        label.setForeground(ColorDefinitions.GENERAL_UNKNOWN_ELEMENT_FOREGROUND_COLOR);
                    }
                }
                return label;
            }
        };
    }

    /**
     * Gets the tool tip for album.
     * 
     * @param a
     *            the a
     * 
     * @return the tool tip for album
     */

    static String getToolTipForAlbum(Album a) {
        int songs = a.getAudioObjects().size();
        return StringUtils.getString(a.getName(), " - ", a.getArtist(), " (", songs, " ", (songs > 1 ? LanguageTool.getString("SONGS") : LanguageTool.getString("SONG")), ")");
    }

    /**
     * Gets the tool tip for artist.
     * 
     * @param a
     *            the a
     * 
     * @return the tool tip for artist
     */

    static String getToolTipForArtist(Artist a) {
        int albums = a.getAlbums().size();
        return StringUtils.getString(a.getName(), " (", albums, " ", (albums > 1 ? LanguageTool.getString("ALBUMS") : LanguageTool.getString("ALBUM")), ")");
    }

    /**
     * Gets the tool tip for genre.
     * 
     * @param g
     *            the g
     * 
     * @return the tool tip for genre
     */

    static String getToolTipForGenre(Genre g) {
        int artists = g.getArtists().size();
        int songs = g.getAudioObjects().size();
        return StringUtils.getString(g.getName(), " (", LanguageTool.getString("ARTISTS"), ": ", artists, ", ", LanguageTool.getString("SONGS"), ": ", songs, ")");
    }

    /**
     * Gets the tool tip for folder
     * 
     * @param f
     *            the folder
     * @return String the tool tip for folder
     */
    static String getToolTipForFolder(Folder f) {
        int songs = f.getAudioFiles().size();
        return StringUtils.getString(f.getName(), " (", songs, " ", (songs > 1 ? LanguageTool.getString("SONGS") : LanguageTool.getString("SONG")), ")");
    }

    /**
     * Gets the tool tip for repository.
     * 
     * @return the tool tip for repository
     */

    static String getToolTipForRepository() {
        int songs = RepositoryHandler.getInstance().getAudioFiles().size();
        return StringUtils.getString(LanguageTool.getString("REPOSITORY"), " (", songs, " ", (songs > 1 ? LanguageTool.getString("SONGS") : LanguageTool.getString("SONG")), ")");
    }

    static Pattern PATTERN = Pattern.compile("(.*)\\s+(.*?)");

    /** The artist names comparator. */
    private Comparator<String> artistNamesComparator = new Comparator<String>() {

        @Override
        public int compare(String s1, String s2) {
            String[] ss1 = s1.split("[,\\&]");
            String[] ss2 = s2.split("[,\\&]");
            String d1 = getStringForSorting(s1, ss1);
            String d2 = getStringForSorting(s2, ss2);
            return collator.compare(d1.toLowerCase(), d2.toLowerCase());
        }

        private String getStringForSorting(String s, String[] ss) {
            String d = "";
            for (String k : ss) {
                Matcher matcher = PATTERN.matcher(k.trim());
                String m = s;
                String n = "";
                if (matcher.matches()) {
                    m = matcher.group(2);
                    n = matcher.group(1);
                }
                d += m + n;
            }
            return d;
        }
    };
}
